"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../../types");
const operation_record_1 = require("../../records/operation.record");
const defaultNotAPaymentObj = {
    userId: 'test-user-id',
    periodId: 'test-period-id',
    isRepetitive: false,
    amount: -45.96,
    type: types_1.OperationType.AddToSavings,
    description: 'user-description',
};
const defaultPaymentObj = {
    userId: "test-user-id",
    periodId: "test-period-id",
    category: types_1.PaymentCategory.BasicNeeds,
    subcategory: types_1.BasicNeeds.Supermarket,
    description: "user-description",
    type: types_1.OperationType.Payment,
    amount: -42.62,
    imgUrl: "/test.jpg",
    isRepetitive: false,
    lat: 49.34342,
    lon: 49.34342,
};
test('OperationEntity can build proper not-a-payment objects.', () => {
    const op = new operation_record_1.OperationRecord(defaultNotAPaymentObj);
    expect(op.userId).toBe('test-user-id');
    expect(op.periodId).toBe('test-period-id');
    expect(op.isRepetitive).toBe(false);
    expect(op.amount).toBe(-45.96);
    expect(op.type).toBe(types_1.OperationType.AddToSavings);
    expect(op.description).toBe('user-description');
});
test('OperationEntity can build proper payment objects.', () => {
    const op = new operation_record_1.OperationRecord(defaultPaymentObj);
    expect(op.userId).toBe('test-user-id');
    expect(op.periodId).toBe('test-period-id');
    expect(op.category).toBe(types_1.PaymentCategory.BasicNeeds);
    expect(op.subcategory).toBe(types_1.BasicNeeds.Supermarket);
    expect(op.description).toBe('user-description');
    expect(op.amount).toBe(-42.62);
    expect(op.imgUrl).toBe('/test.jpg');
    expect(op.isRepetitive).toBe(false);
    expect(op.lat).toBe(49.34342);
    expect(op.lon).toBe(49.34342);
    expect(op.type).toBe(types_1.OperationType.Payment);
});
test('OperationRecord creates a valid UUID for a new entry.', () => {
    const op = new operation_record_1.OperationRecord(defaultPaymentObj);
    expect(op.id).toBeDefined();
    expect(op.id).toMatch(/^[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}$/);
});
test('OperationRecord validates if userId exists', () => {
    expect(() => new operation_record_1.OperationRecord({
        periodId: 'test-period-id',
        userId: '',
        isRepetitive: false,
        amount: -45.96,
        type: types_1.OperationType.AddToSavings,
        description: 'user-description',
    })).toThrow('User ID is required.');
});
test('OperationRecord validates operation type.', () => {
    expect(() => new operation_record_1.OperationRecord({
        userId: 'test-user-id',
        periodId: 'test-period-id',
        isRepetitive: false,
        amount: -45.96,
        description: 'user-description',
    })).toThrow('Wybrano błędny typ kategorii operacji.');
    expect(() => new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { type: 'test' }))).toThrow('Wybrano błędny typ kategorii operacji.');
});
test('OperationRecord validates if isRepetitive exists.', () => {
    expect(() => new operation_record_1.OperationRecord({
        userId: 'test-user-id',
        periodId: 'test-period-id',
        amount: -45.96,
        type: types_1.OperationType.AddToSavings,
        description: 'user-description',
    })).toThrow('isRepetitive is required.');
});
test('OperationRecord validates periodId type.', () => {
    expect(() => new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { periodId: 123 }))).toThrow('periodId should be type of string.');
});
test('OperationRecord validates amount range.', () => {
    expect(() => new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { amount: 9999999999999999999999 }))).toThrow('Kwota operacji powinna zawierać się w przedziale od -999 999 999,99 do 999 999 999,99.');
    expect(() => new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { amount: -9999999999999999999999 }))).toThrow('Kwota operacji powinna zawierać się w przedziale od -999 999 999,99 do 999 999 999,99.');
});
test('OperationRecord validates description.', () => {
    expect(() => new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { description: true }))).toThrow('Opis powinien być tekstem o maksymalnej długości 50 znaków.');
    expect(() => new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { description: '..................................................................................................................................................................................................................................................................................................................................' }))).toThrow('Opis powinien być tekstem o maksymalnej długości 50 znaków.');
});
test('OperationRecord validates payment category.', () => {
    expect(() => new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { category: true }))).toThrow('Dla każdej płatności jej kategoria jest wymagana.');
    expect(() => new operation_record_1.OperationRecord({
        userId: "test-user-id",
        periodId: "test-period-id",
        subcategory: types_1.BasicNeeds.Supermarket,
        description: "user-description",
        amount: -42.62,
        imgUrl: "/test.jpg",
        isRepetitive: false,
        lat: 49.34342,
        lon: 49.34342,
        type: types_1.OperationType.Payment,
    })).toThrow('Dla każdej płatności jej kategoria jest wymagana.');
});
test('OperationRecord validates payment subcategory.', () => {
    expect(() => new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { subcategory: true }))).toThrow('Dla każdej płatności jej podkategoria jest wymagana.');
    expect(() => new operation_record_1.OperationRecord({
        userId: "test-user-id",
        periodId: "test-period-id",
        category: types_1.PaymentCategory.BasicNeeds,
        description: "user-description",
        amount: -42.62,
        imgUrl: "/test.jpg",
        isRepetitive: false,
        lat: 49.34342,
        lon: 49.34342,
        type: types_1.OperationType.Payment,
    })).toThrow('Dla każdej płatności jej podkategoria jest wymagana.');
});
test('OperationRecord validates payment imgUrl.', () => {
    expect(() => new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { imgUrl: true }))).toThrow('Adres URL zdjęcia powinien być tekstem o długości maksymalnie 100 znaków.');
});
test('OperationRecord validates payment imgUrl.', () => {
    expect(() => new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { imgUrl: '...................................................................................................................................................................................................................................................................................................................................................................................' }))).toThrow('Adres URL zdjęcia powinien być tekstem o długości maksymalnie 100 znaków.');
});
test('OperationRecord validates payment coordinates.', () => {
    expect(() => new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { lat: true }))).toThrow('Nie można ustalić lokalizacji.');
    expect(() => new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { lon: true }))).toThrow('Nie można ustalić lokalizacji.');
});
test('OperationRecord throws when object has originId but is not repetitive.', () => {
    expect(() => new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { isRepetitive: false, originId: 'test' }))).toThrow('Operation with origin should be repetitive.');
});
//# sourceMappingURL=operation-record.test.js.map