"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../../types");
const operation_record_1 = require("../../records/operation.record");
const db_1 = require("../../utils/db");
const defaultPaymentObj = {
    userId: "test-user-id",
    periodId: "test-period-id",
    category: types_1.PaymentCategory.BasicNeeds,
    subcategory: types_1.BasicNeeds.Supermarket,
    description: "user-description",
    type: types_1.OperationType.Payment,
    amount: -42.62,
    imgUrl: "/test.jpg",
    isRepetitive: false,
    lat: 49.34342,
    lon: 49.34342,
};
const defaultOperationEntity = new operation_record_1.OperationRecord({
    id: '[TEST]',
    userId: "testUserId",
    type: 2,
    description: 'test description',
    isRepetitive: false,
    amount: 99,
});
beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
    yield defaultOperationEntity.insert();
}));
afterAll(() => __awaiter(void 0, void 0, void 0, function* () {
    yield defaultOperationEntity.delete();
    yield db_1.pool.end();
}));
test('OperationRecord returns data from database for one entry.', () => __awaiter(void 0, void 0, void 0, function* () {
    const op = yield operation_record_1.OperationRecord.getOne('[TEST]', 'testUserId');
    expect(op).toBeDefined();
    expect(op.id).toBe('[TEST]');
    expect(op.userId).toBe('testUserId');
    expect(op.type).toBe(2);
    expect(op.isRepetitive).toBe(false);
    expect(op.amount).toBe(99.00);
}));
test('OperationRecord returns null for non-existing entry.', () => __awaiter(void 0, void 0, void 0, function* () {
    const op = yield operation_record_1.OperationRecord.getOne('[non-existing-entry]', 'testUserId');
    expect(op).toBeNull();
}));
test('OperationRecord returns null when operation ID is valid, but userID is wrong.', () => __awaiter(void 0, void 0, void 0, function* () {
    const op = yield operation_record_1.OperationRecord.getOne('[TEST]', 'wrong-user-id');
    expect(op).toBeNull();
}));
test('OperationRecord.getAll returns an array of found entries.', () => __awaiter(void 0, void 0, void 0, function* () {
    const opArray = yield operation_record_1.OperationRecord.getAll('', 'testUserId');
    expect(opArray).not.toEqual([]);
    expect(opArray[0].id).toBeDefined();
}));
test('OperationRecord.getAll returns an array of found entries when searching for "e".', () => __awaiter(void 0, void 0, void 0, function* () {
    const opArray = yield operation_record_1.OperationRecord.getAll('e', 'testUserId');
    expect(opArray).not.toEqual([]);
    expect(opArray[0].id).toBeDefined();
}));
test('OperationRecord.getAll returns empty array of found entries when userId is wrong.', () => __awaiter(void 0, void 0, void 0, function* () {
    const opArray = yield operation_record_1.OperationRecord.getAll('e', 'non-existing-userId');
    expect(opArray).toEqual([]);
}));
test('OperationRecord.getAll returns empty array of found entries when searching that does not exist.', () => __awaiter(void 0, void 0, void 0, function* () {
    const opArray = yield operation_record_1.OperationRecord.getAll('non-existing-description', 'testUserId');
    expect(opArray).toEqual([]);
}));
test('OperationRecord.insert returns new UUID and inserts data into database. OperationRecord.delete can remove this entry from database.', () => __awaiter(void 0, void 0, void 0, function* () {
    const op = new operation_record_1.OperationRecord(defaultPaymentObj);
    const newOpId = yield op.insert();
    expect(op).toBeDefined();
    expect(op.id).toBeDefined();
    expect(op.id).toBe(newOpId);
    expect(op.id).toMatch(/^[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}$/);
    yield op.delete();
    const notFoundOp = yield operation_record_1.OperationRecord.getOne(op.id, 'test-user-id');
    expect(notFoundOp).toBeNull();
}));
test('OperationRecord.update changes data in database.', () => __awaiter(void 0, void 0, void 0, function* () {
    const op = new operation_record_1.OperationRecord(defaultPaymentObj);
    yield op.insert();
    expect(op.description).toBe("user-description");
    op.category = types_1.PaymentCategory.Additional;
    op.subcategory = types_1.Additional.Taxis;
    op.description = 'New description';
    op.type = types_1.OperationType.Payment;
    op.amount = -45453.34;
    op.imgUrl = 'https://newImgUrl.com';
    op.isRepetitive = true;
    op.lat = 54.34342;
    op.lon = 34.34234;
    const opId = yield op.update();
    expect(opId).toBe(op.id);
    expect(op.category).toBe(1);
    expect(op.subcategory).toBe(5);
    expect(op.description).toBe('New description');
    expect(op.type).toBe(1);
    expect(op.amount).toBe(-45453.34);
    expect(op.imgUrl).toBe('https://newImgUrl.com');
    expect(op.isRepetitive).toBe(true);
    expect(op.lat).toBe(54.34342);
    expect(op.lon).toBe(34.34234);
    yield op.delete();
}));
test(`OperationRecord.findPeriodOperations should find all operations of given user's period`, () => __awaiter(void 0, void 0, void 0, function* () {
    const defaultOperationObj = Object.assign(Object.assign({}, defaultPaymentObj), { userId: '[TEST-USER]5', periodId: '[TEST-PERIOD]5' });
    const operationOne = new operation_record_1.OperationRecord(Object.assign({}, defaultOperationObj));
    const operationTwo = new operation_record_1.OperationRecord(Object.assign({}, defaultOperationObj));
    yield operationOne.insert();
    yield operationTwo.insert();
    const foundOperations = yield operation_record_1.OperationRecord.findPeriodOperations(defaultOperationObj.periodId, defaultOperationObj.userId);
    expect(foundOperations).toBeDefined();
    expect(foundOperations.length).toBe(2);
    yield operationOne.delete();
    yield operationTwo.delete();
}));
test(`OperationRecord.findRepetitiveOperations finds all user's repetitive operations`, () => __awaiter(void 0, void 0, void 0, function* () {
    const rootOp = new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { description: 'test-root-op-1', periodId: null, isRepetitive: true }));
    const secondRootOp = new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultPaymentObj), { description: 'test-root-op-2', periodId: null, isRepetitive: true }));
    yield rootOp.insert();
    yield secondRootOp.insert();
    const found = yield operation_record_1.OperationRecord.findRepetitiveOperations('test-user-id');
    expect(found.length).toBe(2);
    yield rootOp.delete();
    yield secondRootOp.delete();
}));
//# sourceMappingURL=operation.test.js.map